function [ nPos, j ] = nsect(strFn, nPos, nSlope, nAp, varargin);

% NSECT call the correct functions to do the tracing.  Do the 
% tracing on the canonical version of the surface. 
% 
% [nPos, j] = NSECT(strFn, nPos, nSlope, nAp, ...). Trace the ray with 
% with position nPos and nSlope to the surface defined by strFn.  The 
% rays that fall outside the aperture nAp will be discarded. 
%
% This function assumes the surface centered frame. 
%
%  EXAMPLE 
%
%   % trace a conic in the canonical frame 
%   CV = 1/7; CC = 0;  % spherical conic
%   [y, j] = nsect('conic', [0 1 -1], [ 0 0 1], 3, CV, CC);
%   y, j
% 



% NB : 
%       added extra code to perform a general ray-trace 
%       if the analytic ray-trace failed for some reason. 
%


   strFn0 = sprintf('trace_%s', strFn);
   nPos0 = nPos; nSlope0 = nSlope;
   [nPos, j] = feval(strFn0, nPos, nSlope, nAp, varargin{:});

   % consistency tolerance
   etol = 1e-7;  % consistency tolerance 
   
   if (~isempty(nPos)),
    
        % consistency testing
        [y, i, e] = isconsistent(strFn, nPos, nSlope, etol, varargin{:});
        if (y),
            % all consistent 
            return
        end 
        
        error('INCONSISTENCY DETECTED.');
                
        % fix for inconsistency
        % [nPos(i,:), j(i)] = trace_general(strFn, nPos0(i,:), nSlope(i,:), nAp, etol, varargin{:});                        
        [nPos1, i0] = trace_general(strFn, nPos0(i,:), nSlope(i,:), nAp, etol, varargin{:});        
                
        j0 = j(i);           % actual indexes tested
        j0 = j0(i0);         % actual indexes passed 
        nPos(j0, :) = nPos1; 
        keyboard;
        
        [y, i, e] = isconsistent(strFn, nPos(i,:), nSlope(i,:), etol,varargin{:});
        if (y),
            % all consistent 
            return
        end



        error('(NSECT) Inconsistent ray-traces occured. check inputs make sense.');
        
    end 
   
   
   
   
return


function [y, i, e] = isconsistent(strFn, nPos, nSlope, etol, varargin),


   strFn0 = sprintf('trace_%s', strFn);

   % ray trace check - was intersection consistent? 
   switch(feval(strFn0,'type')),
      case { 'analytic' }
         z0 = feval(strFn, nPos(:,1), nPos(:,2), varargin{:});
      case { 'numeric' }
         z0 = feval(strFn, nPos(:,1), nPos(:,2), varargin{2:end});
      otherwise,
         error('(NSECT) Unknown analytic or numeric.');
   end 
      
                
   e  = abs(z0-nPos(:,3));
   i  = (e > etol);
   if (any(i)),
        y = 0;
        return
   end     

   y =1;
   i = [];
   
return 
