function [m_nPos, m_j] = trace_conic(nPos, nSlope, nAp, CV, CC)

% TRACE_CONIC intersect a origin centered conic surface
% with a ray (nPos, nSlope). 
%
% TRACE_CONIC(nPos, nSlope, nAp, CV, CC).  Here nPos is the position,
% nSlope is the ray slope, nAp is the aperture and (CV, CC) are conic 
% arguments. The intersection point is returned as well as an index 
% into the rays that are remaining.
% 
% NOTE : 
%       R  = radius of curvature 
%       CC = conic constant (-e^2) 
%               CC = 0  SPHERE, 
%               CC = -1 PARABOLOID, 
%               CC < -1 HYPERBOLOID, 
%          -1 < CC < 0  ELLIPSOID  
%                       
%       info.aperture   = aperture
%       info.sense      = 'concave','convex'
%
% 
% NOTE : 
%       1/ root detection tries to account for every possibility.
%       2/ note that if a ray starts on a surface the trace will succeed. 
%          regardless of ray direction/slope 
%       3/ assumes R ~= 0, nSlope(:,3) = 0
%       4/ added consistency detection


% 
% "Corneal surface reconstruction that uses Zernike polynomial
% representation" Sicam et. al. JOSA A, 21(7), 2004.
%
% based on eqn. (1) [ s^2 - 2rz + kz^2 = 0 ]   
% where k = 1 - e^2 and e is eccentricity 
%
% verified against, OSLO manual
% http://www.lambdares.com/pub/Optics_Reference.pdf
% 

if (nargin == 1),
    if (strcmp(nPos, 'type')),
        m_nPos = 'analytic'; m_j = [];
        return
    end
    error('(TRACE_CONIC) Error in invocation.');
end

j   = (1:size(nPos, 1))';  
m_j = []; m_nPos = [];

% main intersection 
[l, q ] = getConicRoot(nPos, nSlope, CV, CC);
if (isempty(l)),
    % no valid rays
    return
end

nPos = nPos(q, :); nSlope = nSlope(q, :); j = j(q);

% send to surface 
l    = l(:,[1 1 1]);
nPos = nSlope.*l + nPos;
r    = sqrt( nPos(:,1).^2 + nPos(:,2).^2 );
ndx  = (r <= nAp);
if   any(ndx),
      m_j      = j(ndx);
      m_nPos   = nPos(ndx,:);
      m_nSlope = nSlope(ndx,:);
        
%      % partial safety check
%      z0 = conic(m_nPos(:,1), m_nPos(:,2), CV, CC);
%      e  = abs(z0-m_nPos(:,3));
%      i  = (e > 1e-6);    
%      if (any(i)),
%          error('(TRACE_CONIC) It appears an inconsistent ray trace occured.');    
%      end 
      return
end

% disp('how did I get here??');
% keyboard;

% remaining rays are good.
error('(TRACETO_CONIC) Error. Rays fall outside aperture.');
return

% GETCONICROOT return a valid root. Attempt to detect all 
% types of intersections. This function does "sorting" 
% in place, i.e., avoids having to use SORT (I don't like it)

function [l, j] = getConicRoot(nPos, nSlope, CV, CC),

k   = CC + 1;       % k = CC + 1 where CC = -e^2
r   = 1/CV;         % r = R 

j   = (1:size(nPos, 1))';  
x0  = nPos(:,1); y0 = nPos(:,2); z0 = nPos(:,3);
u0  = nSlope(:,1); v0 = nSlope(:,2); w0 = nSlope(:,3);

% determine intersection coefficients
a   = (u0.^2 + v0.^2) + (CC+1)*w0.^2;
b   = 2*(u0.*x0 + v0.*y0 - w0/CV + (CC+1)*w0.*z0);
c   = x0.^2 + y0.^2 + (CC+1)*z0.^2 - 2*z0/CV;

ndx = (a ~= 0);
if (any(ndx)), % double root 
    j0 = find(ndx);
    d  = b(ndx).^2 - 4*a(ndx).*c(ndx);
    
    j1 = (d < 0);
    if (any(j1)), 
       if (all(j1)),
           l = []; j = [];
           return
       end 
       
        % some rays missed
        l(j0(j1)) = nan;  % missed rays
    end

    % remaining rays
    j1 = ~j1; n0 = j0(j1);
    l1 = (-b(n0) + sqrt(d(j1)))./(2*a(n0));
    l2 = (-b(n0) - sqrt(d(j1)))./(2*a(n0));    
    
    % discard - 2 negative roots 
    j1 = (l1 < 0)&(l2 < 0);
    if (any(j1)),
        l(j0(j1)) = nan;    
    end
    
    % accept l1 root 
    j1 = (l2 < 0)&(l1 >= 0);
    if (any(j1)),
        l(j0(j1)) = l1(j1);    
    end
        
    % accept l2 root 
    j1 = (l1 < 0)&(l2 >= 0);
    if (any(j1)),
        l(j0(j1)) = l2(j1);    
    end
    
    % two positive roots
    j1 = (l1 >=0)&(l2 >=0);
    if (any(j1)),
        w0 = nSlope(j0(j1), 3); 
        j1  = (w0*r > 0);
        if (CC >= -1),
          if (any(j1)), % min root
            l(j0(j1))  = l2(j1); 
          end
          j1 = ~j1;
          if (any(j1)), % max root
            l(j0(j1))  = l1(j1); 
          end
        else 
          if (any(j1)), % max. root
            l(j0(j1))  = l1(j1); 
          end
          j1 = ~j1;
          if (any(j1)), % min root
            l(j0(j1))  = l2(j1); 
          end
        end 
    end
    
end 

ndx = ~ndx;
if (any(ndx)), % single root
    l(ndx) = -c(ndx)./b(ndx);
end 


% final cleanup
l = l(:);
ndx = (isfinite(l)&(l>=0));
if (any(ndx)),
    l = l(ndx); 
    j = j(ndx);
    return
end 

l = []; j = [];
return