function [m_nPos, m_j] = trace_ellipsoid(nPos, nSlope, nAp, a, b, c)

% TRACE_ELLIPSOID intersect a origin centered conic surface
% with a ray (nPos, nSlope). 
%
% TRACE_ELLIPSOID(nPos, nSlope, nAp, A, B, C).  Here nPos is the position,
% nSlope is the ray slope, nAp is the aperture and (A, B, C) are the  
% semi-axes of the ellipsoid. The intersection point is returned as well as an index 
% into the rays that are remaining.
%
%
% NOTE : 
%       1/ root detection tries to account for every possibility.
%       2/ note that if a ray starts on a surface the trace will succeed. 
%          regardless of ray direction/slope 
%       3/ added consistency detection


if (nargin == 1),
    if (strcmp(nPos, 'type')),
        m_nPos = 'analytic'; m_j = [];
        return
    end
    error('(TRACE_ELLIPSOID) Error in invocation.');
end

if (nargin == 1),
    if (strcmp(nPos, 'type')),
        m_nPos = 'analytic'; m_j = [];
        return
    end
    error('(TRACE_ELLIPSOID) Error in invocation.');
end

j   = (1:size(nPos, 1))';  
m_j = []; m_nPos = [];

% main intersection 
[l, q ] = getElpRoot(nPos, nSlope, a, b, c);
if (isempty(l)),
    % no valid rays
    return
end

nPos = nPos(q, :); nSlope = nSlope(q, :); j = j(q);

% send to surface 
l    = l(:,[1 1 1]);
nPos = nSlope.*l + nPos;
r    = sqrt( nPos(:,1).^2 + nPos(:,2).^2 );
ndx  = (r <= nAp);
if   any(ndx),
     if (all(ndx)),
        m_j      = j(ndx);
        m_nPos   = nPos(ndx,:);
        m_nSlope = nSlope(ndx,:);
        
        % partial safety check
        z0 = ellipsoid(m_nPos(:,1), m_nPos(:,2), a, b, c);
        e  = abs(z0-m_nPos(:,3));
        i  = (e > 1e-6);
        if (any(i)),
            error('(TRACE_ELLIPSOID) It appears an inconsistent ray trace occured.');    
        end 
               
        
        return
     end 
     
end

% remaining rays are good.
error('(TRACETO_ELLIPSOID) Error. Rays fall outside aperture.');
return

% GETCONICROOT return a valid root. Attempt to detect all 
% types of intersections. This function does "sorting" 
% in place, i.e., avoids having to use SORT (I don't like it)

function [l, j] = getElpRoot(nPos, nSlope, a0, b0, c0),

j   = (1:size(nPos, 1))';  
X0 = nPos; M = nSlope; DXC = X0; 
DXC(:,3) = DXC(:,3) - c0;

A  = [ a0 b0 c0 ];
A  = (1./(A.^2))';
a  = (M.*M)*A;
b  = 2*(M.*DXC)*A;
c  = (DXC.*DXC)*A - 1;
d   = b.^2 - 4*a.*c;
   
j1 = (d < 0);
if (any(j1)), 
   if (all(j1)),
      l = []; j = [];
      return
   end 
   
   % to discard
   l(j(j1)) = nan;
   
   % remaining
   j1 = ~j1; j = j(j1);
   a = a(j1); b = b(j1);
   c = c(j1); d = d(j1);
end

l1 = (-b + sqrt(d))./(2*a);
l2 = (-b - sqrt(d))./(2*a);    

% discard - 2 negative roots 
j1 = (l1 < 0)&(l2 < 0);
if (any(j1)),
    if (all(j1)),
      l = []; j = [];
      return        
    end
    
   % to discard
   l(j(j1)) = nan;
   
   % remaining
   l1 = l1(j1); l2 = l2(j1);
   j  = j(j1);   
   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% intersections detected 

% accept l1 root 
j1 = (l2 < 0)&(l1 >= 0);
if (any(j1)),
    l(j(j1)) = l1(j1);    
end
        
% accept l2 root 
j1 = (l1 < 0)&(l2 >= 0);
if (any(j1)),
   l(j(j1)) = l2(j1);    
end
    
% two positive roots
j1 = (l1 >=0)&(l2 >=0);
if (any(j1)),
        
    w0 = nSlope(j(j1), 3); 
    j1  = (w0*c0 > 0);
    if (any(j1)), % min root
       l(j(j1))  = l2(j1); 
    end
    
    j1 = ~j1;
    if (any(j1)), % max root
       l(j(j1))  = l1(j1); 
    end
end 

% cleanup
l = l(:);
ndx = (isfinite(l)&(l>=0));
if (any(ndx)),
    l = l(ndx); 
    j = j(ndx);
    return
end 

error('(TRACE_ELLIPSOID) Programming error. No valid traces found - should be at least one.'); 
return