function [nPos, j] = trace_general(strFn, nPos, nSlope, nAp, etol, varargin)

% TRACE_GENERAL intersect any ray (nPos, nSlope) surface that 
% has a centered representation using a numeric approach. 
%
% TRACE_GENERAL(strFn, nPos, nSlope, nAp, ...).  Here nPos is the position,
% nSlope is the ray slope, nAp is the aperture and additional arguments 
% are passed on the command line. The intersection point is returned as well 
% as an index into the rays that are remaining.
% 
% 

[s, j] = general(@m_gen, nPos, nSlope, nAp, etol, strFn, varargin{:});

k = ~isnan(s);
if (any(k)),
    j = j(k); s = s(k);
    nPos = nPos(k, :); nSlope = nSlope(k, :);
    nPos(:,1) = nPos(:,1) + nSlope(:,1).*s;
    nPos(:,2) = nPos(:,2) + nSlope(:,2).*s;
    nPos(:,3) = nPos(:,3) + nSlope(:,3).*s;
    return
end

% nothing to return 
nPos = []; j = [];
return 


function e = m_gen(s, x0, nAp, varargin),

% M_GEN Objective fn. for ray-tracing
%
% M_GEN(s, nPos, nSlope, nAp, ...). Return the signed error 
% [ error is -VE before the surf. and +VE if after the surf. ]. 
%

strFn = varargin{1};
nPos   = x0(:,1:3);
nSlope = x0(:,4:6);

s    = repmat(s, 1, 3);  
nPos = nPos + s.*nSlope;
z0   = feval(strFn, nPos(:,1), nPos(:,2), varargin{2:end});   % surface at given point 

% error [ -VE Prior Surf., +VE Post. Surf. ]
e    = (nPos(:,3) - z0); 
e    = sign(nSlope(:,3)).*e;  

% set outside aperture 
rp = sqrt(nPos(:,1).^2 + nPos(:,2).^2);    
i  = (rp > nAp);
if (any(i)),
   e(i) = nan;
end
return


function [m_s, m_nd] = general(strObjFn, nPos, nSlope, nAp, etol, varargin);

% GENERAL.  Objective function should cross through zero.
% 

        nd = (1:size(nPos,1))';
        
        % Find bounding cylinder intersections 
        %
        %    a = nan & b=nan   no intersection i.e. missed or coaxial 
        %    a = 0   & b = 0   one intersection point only
        %    a = 0   & b > 0   ray points away (not through) from bound 
        %    a > 0   & b > 0   two cylinder      
        
        % detect any parallel rays              
        r = sqrt(nSlope(:,1).^2 + nSlope(:,2).^2);
        i = (r == 0);
        if (any(i)),
           fprintf('(GENERAL) parallel rays.\n');
           strFn = varargin{1};
           z0  = feval(strFn, nPos(i,1), nPos(i,2), varargin{2:end});
           s0  = abs(z0 - nPos(i,3));
           % upload results
           m_s(nd(i))  = s0;
           m_nd(nd(i)) = nd(i); 
           % early return
           if (all(i)),
                m_nd = m_nd(:); m_s = m_s(:);
                return
            end 
            % get the remainder
            i  = ~i; nd = nd(i);
            a = a(i); b = b(i);
            nPos = nPos(i,:); nSlope = nSlope(i,:);
            
            keyboard;
            
        end
        
        % attempt to get valid intervals (a, b).
        [a, b] = getCylBnd(nPos, nSlope, nAp, varargin{:});
        
        
        fprintf('GET CYLBND.\n');
        keyboard;
        
        
        i = (isnan(a)&isnan(b));
        if (any(i)), % upload results
            fprintf('(GENERAL) GETCYLBND returned NAN.\n');
            m_s(nd(i))  = nan;
            m_nd(nd(i)) = nd(i); 
            if (all(i)),
                m_nd = m_nd(:); m_s = m_s(:);
                return
            end
            % get the remainder
            i  = ~i; nd = nd(i);
            a = a(i); b = b(i);
            nPos = nPos(i,:); nSlope = nSlope(i,:);            
        end
        
        % attempt small adjust if finite (a, b) but (f(a), f(b))
        [a, b, fa, fb] = getValidInt(strObjFn, a(:), b(:), [ nPos nSlope ], nAp, varargin{:});
        if (isnan(fa.*fb)),
           error('(GENERAL) GETVALIDINT couldnt find non-finite (f(a), f(b)).');
        end 
        
        keyboard;
        
%        disp('GETVALIDINT');
%        [ a b fa fb ]
        
        % valid but seem to miss the aperture
        i = (fa.*fb > 0);
        if (any(i)),
          m_s(nd(i))  = nan;
          m_nd(nd(i)) = nd(i); 
          if (all(i)), % early return
              fprintf('(GENERAL) Rays MISS aperture.\n');
              m_nd = m_nd(:); m_s = m_s(:);
              return
          end 
            
          % get the remainder
          i = ~i; nd = nd(i);
          a = a(i); b = b(i);
          nPos = nPos(i,:); nSlope = nSlope(i,:);            
        end
                
        fprintf('(GENERAL) Drop to VSECANT search.\n');
        
        size(a)
        size(b)
        
        
        % valid bounds (a, b) & (f(a), f(b))
        [s0, e] = vsecant(strObjFn, a(:), b(:), etol, [ nPos nSlope ], nAp, varargin{:});
        
        size(a)
        size(b)
        
        
        i  = (isreal(s0)&(s0 > 0));
        if (any(i)),        
            disp('GETVALIDINT');
%           s0 
            % upload into final
            m_s(nd(i))  = s0(i);
            m_nd(nd(i)) = nd(i); 
            
            fprintf('waiting ...');
            keyboard;
            
            
            if (all(i)), % early return
                fprintf('(GENERAL) done OK.\n');
                m_nd = m_nd(:); m_s = m_s(:);
%               fprintf('END');
%               keyboard;
                return
            end 
            
            % get the remainder
            i = ~i; nd = nd(i);
            a = a(i); b = b(i);
            nPos = nPos(i,:); nSlope = nSlope(i,:);            
            s0 = s0(i);
        end
  
        warning('(GENERAL) Dropping to 2nd Phase Search.');
        
        
        % stepped outside of limits
        i = isnan(s0);
        if (any(i)), 
            fprintf('(GENERAL) 2ND PHASE : drop to VBRENT.\n');
            s0 = vbrent(strObjFn, a(:), b(:), etol, [ nPos nSlope ], varargin{:});
            m_s(nd(i))  = s0;
            m_nd(nd(i)) = nd(i); 
            
            if (all(i)), % early return
                m_nd = m_nd(:); m_s = m_s(:);   
                return
                return
            end 
                        
            % reduce these 
            i = ~i; nd = nd(i);
            a = a(i); b = b(i);
            nPos = nPos(i,:); nSlope = nSlope(i,:);            
            s0 = s0(i);
        end 
        
        error('(GENERAL) Unprocessed Rays. REQUIRE FIXING.');
return
return 

