function n = herzberger(x, n1, nF, nc, n2),

% HERZBERGER. Implement the herzberger formula 
%
% HERZBERGER(x, n**, nF, nc, n*). Here x is the wavelength
% in micrometers, (n**, nF, nc, n*) are the coefficients as
% described below. 
% 
% HERZBERGER(x, [n**, nF, nc, n*]). Here x is the wavelength
% in micrometers, (n**, nF, nc, n*) is a vector of the same
% coefficients.
%
% The Herzberger formula is given by 
%
%  n(l) = a1(l)(n**) + a2(x)(nF) + a3(x)(ne) + a4(x)(n*)
%
% where 
% 
%   n** = n(x = 0.365 micron)
%   nF  = n(x = 0.4861 micron)
%   nc  = n(x = 0.6563 micron)
%   n*  = n(x = 1.014 micron)
%
%   Medium    n**     nF      nc      n* 
%   -----------------------------------------
%   Cornea    1.3975  1.3807  1.37405 1.3668
%   Aqueous   1.3593  1.3422  1.3354  1.3278
%   Lens      1.4492  1.42625 1.4175  1.4097
%   Vitreous  1.3565  1.3407  1.3341  1.3273
%
% The a1, a2, a3, a4 factors are given by, 
%
%   a1 = A0 + A1*w^2 + P/(w^2-w0^2) + R/(w^2-w0^2)^2
%
% where (corrected data from Smith, G. and Atchison, DA. (2005)) 
%
%      A0           A1            P             R
% -----------------------------------------------------
%  0.66147196  -0.40352796  -0.2804679    0.03385979  
% -4.20146383   2.73508956    1.50543784  -0.11593235  
%  6.29834237  -4.69409935   -1.5750865    0.10293038  
% -1.75835059   2.3625379     0.35011657  -0.02085782 
%
%
% using details presented in :
% 
% R. Navarro, J. Santamaria, and J. Bescos, "Accommodation-dependent model of the 
% human eye with aspherics," J. Opt. Soc. Am. A 2(8), 1273-1281 (1985)
% 
% Corrections by Smith, G. and Atchison, DA. "Chromatic dispersions of the ocular 
% media of human eyes." JOSA A (2005), 22(1), 29 -- 37. 
%
% 
% Example, [ compare errors at 900 nm ]
% 
%   % cf. two methods at 900 nm (herzberger (micron)/ cauchy (nm))
%   % Smith and Atchison suppose a 0.003 diff. 
%   nh = herzberger(900/1000, 1.3975, 1.3807, 1.37405, 1.3668);
%   nc = cauchy(900, 1.362017, 7.001974e3, -1.020784e9, 9.755871e13);
%   fprintf('NH = %6.6f  NC = %6.6f DF = %6.6f\n', nh, nc, nc-nh);
%


M = [ ... 
 0.66147196  -0.40352796  -0.2804679   0.03385979 ; ... 
 -4.20146383  2.73508956   1.50543784  -0.11593235 ; ... 
 6.29834237  -4.69409935   -1.5750865   0.10293038 ; ... 
 -1.75835059  2.3625379    0.35011657  -0.02085782 ];
x0 = 0.1673; % micron

w = [ 1 x.^2 1./(x.^2-x0.^2) 1./(x.^2-x0.^2).^2 ]'; 
a = M*w;  
n = n1.*a(1) + nF.*a(2) + nc.*a(3)  + n2.*a(4); 

return
