 function x = parax(p),

% PARAX return a structure with paraxial imaging properties if 
% possible.
%
% PARAX(s), Here s is the optic structure.
% 
% NOTE : 
%
% FVP : front vertex point (FVP) 
% VF1 : FVP to 1st focal point (FP) 
% PF1 : PPL1 to 1st FP
% VP1 : FVP to PPL1 (1st Principal Plane)
% VN1 : FVP to N1 (1st nodal poinr)
% BVP : back vertex point 
% VF2 : BVP to 2nd FP 
% PF2 : PPL2 to 2nd FP
% VP2 : BVP to PPL2
% VN2 : BVP to N2
% 
% These calculations can be used to determine cardinal points 
% with respect to the front vertex plane/point. 
% 
% P1 = VP1;             % 1st PPL plane 
% P2 = BVP + VP2;       % 2nd PPL plane 
% N1 = FVP + VN1;       % 1st nodal point
% N2 = BVP + VN2;       % 2nd nodal point
% F1 = FVP + VF1;       % 1st focal point 
% F2 = BVP + VF2;       % 2nd focal point 
% 
% Based on,
%
%  Appendix 1 : Advanced paraxial optics 
%  The Eye and Visual Optical Instruments
%  Smith, G. and Atchison, D.A. 
%  Cambrdige University Press, 1997.
%
%
%   [uj']   [ nj/nj' -Fj/nj' ][uj]
%   [   ] = [                ][  ]
%   [hj']   [ 0         1    ][hj]
% 
%   where 
%           uj, uj' = input/output slope 
%           hj, hj' = input/output slope 
%           nj/nj'  = input/output ref. index
%           Fj      = surface power Cj(nj'-nj)
%           Cj      = 1/R



% return the system matrix 
S  = eye(2);

% initialize
p   = gohead(p);
n1  = get(p, 'args', 'index');
p   = next(p);
R   = getR(p);   % R from PRIM 

z1  = get( get(p), 'position'); z1 = z1(3);
z0  = z1; z2 = z1;
while   (1),
        
    p   = next(p);
    n2  = get(p, 'args', 'index');
    F  = (1/R)*(n2-n1);   
    R  = [ n1/n2 -F/n2 ; 0 1 ];        
    S  = R*S;
       
    % refraction step
    p  = next(p);

    % stop if last surface reached.
    if (islast(p)),
        break
    end 
    
    R = getR(p);   % R from PRIM 
    z2 = get( get(p), 'position'); z2 = z2(3);    

    
    % transfer step
    d  = z2-z1;
    T  = [ 1 0 ; d 1 ]; 
    S  = T*S;
    
    n1 = n2; z1 = z2;
end


A = S(1,1); B = S(1,2);
C = S(2,1); D = S(2,2);

% system matrix
x.S   = S;

% cardinal points 
x.FVP = z0;           % front vertex point (FVP) 
x.VF1 = A/B;          % FVP to 1st FP 
x.PF1 = D*A/B-C;      % PPL1 to 1st FP
x.VP1 = C+(1-D)*A/B;  % FVP to PPL1
x.VN1 = (A-1)/B;      % FVP to N1

x.BVP = z2;           % back vertex point (BVP)
x.VF2 = -D/B;         % BVP to 2nd FP 
x.PF2 = -1/B;         % PPL2 to 2nd FP
x.VP2 = (1-D)/B;      % BVP to PPL2
x.VN2 = D*(A-1)/B-C;  % BVP to N2

% all with respect to the front vertex point
x.P1 = x.VP1;
x.P2 = x.BVP + x.VP2;
x.N1 = x.FVP + x.VN1;
x.N2 = x.BVP + x.VN2;
x.F1 = x.FVP + x.VF1;
x.F2 = x.BVP + x.VF2;

% fprintf('--------------------------\n');
% fprintf('front vertex point = %2.6f\n',x.FVP);
% fprintf('FVP to FP1         = %2.6f\n',x.VF1);
% fprintf('PPL1 to FP1        = %2.6f\n',x.PF1);
% fprintf('FVP to PPL1        = %2.6f\n',x.VP1);
% fprintf('FVP to N1          = %2.6f\n',x.VN1);
% fprintf('--------------------------\n');
% fprintf('back vertex point  = %2.6f\n',x.BVP);
% fprintf('BVP to FP2         = %2.6f\n',x.VF2);
% fprintf('PPL2 to FP2        = %2.6f\n',x.PF2);
% fprintf('BVP to PPL2        = %2.6f\n',x.VP2);
% fprintf('BVP to N2          = %2.6f\n',x.VN2);
% fprintf('--------------------------\n');
% fprintf('P1        = %2.6f\n',x.P1);
% fprintf('N1        = %2.6f\n',x.N1);
% fprintf('F1        = %2.6f\n',x.F1);
% fprintf('P2        = %2.6f\n',x.P2);
% fprintf('N2        = %2.6f\n',x.N2);
% fprintf('F2        = %2.6f\n',x.F2);
% fprintf('--------------------------\n');
return 

function R = getR(p),
    
    p = get(p); % current PRIM
    
    switch(get(p,'type')),
        case { 'conic' }
            R  = 1/get(p, 'args', 'CV'); 
        case { 'sphere' } 
            R  = get(p, 'args', 'R'); 
        case { 'pupil','img' }
            R  = inf;
        otherwise,
            error('(PARAX) Cannot determine R for this SURFACE.');
    end
return