function [f, fx, fy] = pfn2dmtf(w, pupil, nD, nR, nLambda, nIndex),

% PFN2DMTF Obtain the Diffractiom Modulation Transfer Function (DMTF)
% starting from the complex pupil function. Pad the complex pupil  
% function with 2*size(w). 
% 
% [f, fx, fy] = PFN2DMTF(w, pupil, nD, nR, Lambda, nIndex). The variable w 
% is the wavefront error (in wave units), pupil is the mask. Returns the MTF 
% in f(fx, fy) with (fx, fy) the image space frequency variables in cyc/deg (CPD). 
% 
% Extra information required,
%
%   nLambda   Wavelength (Image Space)          
%   nIndex    Image space index     
%   nD        Exit pupil diameter        
%   nR        Radius of reference sphere
%
% EXAMPLE, 
%
% Approximate MTF of the eye (3 mm exit pupil).
%
%   [w, pupil] = pfnAiry(128, 1);    % pupil 
%   nLambda = 0.5893e-3; nIndex = 1; nR = 20; nD = 3;
%   [f, fx, fy] = pfn2dmtf(w, pupil, nD, nR, nLambda, nIndex);
%   plot_dmtf(fy(:,1), f(:,1));
%
% DMTF for the Navarro eye (4 mm aperture STOP).

%   
% NOTES : 
%
%   Pupil function metrics adapted from :
%  
%   Dr. John S. Loomis
%   Associate Professor, Electrical and Computer Engineering
%   University of Dayton
%   Dayton, OH 45469-0226
%   Office: KL-363
%   Phone: (937) 229-3981
%   EMail: John.Loomis@notes.udayton.edu
% 
%   http://www.engr.udayton.edu/faculty/jloomis/
% 

w       = pupil.*exp(2*pi*j*w);
f       = acorr2(w);
f       = abs(f/max(f(:)));

if (nargin == 2),
    fx = []; fy = [];
    return
end


% scaling factors 
M       = size(w, 1);            % PFN GRID (rays traced)

% exit pupil 
dY1     = nD/M;                  % exit pupil cell size
Fs1     = 1/dY1;                 % Sampling rate 
Fc1     = 1/(2*dY1);             % Cutoff freq. (exit pupil) 

N       = 2*M;                   % FFT grid sampling  (NOT USED).
dY2     = (nLambda*nR)/(N*dY1);  % image plane cell size  
Fc2     = 1/(2*dY2);             % Cutoff freq. (image plane)

theta   = radtodeg(1/nR);           % 1mm / 1deg          
% Fc2     = Fc2/theta;

% MTF scale factors
fx = (Fc2)*linspace(0, 1, M);
fx = repmat(fx, M, 1);
fy = fx'; f = f(1:M, 1:M);

f  = rot90(rot90(f));
return